'use strict';

function _typeof(obj) { return obj && typeof Symbol !== "undefined" && obj.constructor === Symbol ? "symbol" : typeof obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var wrap = require('../index');
var lifeCycleMethods = require('../lifeCycleMethods');
var assert = require('assert');
var React = require('react');
var ReactTestUtils = require('react-addons-test-utils');

var EmptyMockComponent = (function (_React$Component) {
    _inherits(EmptyMockComponent, _React$Component);

    function EmptyMockComponent() {
        _classCallCheck(this, EmptyMockComponent);

        return _possibleConstructorReturn(this, Object.getPrototypeOf(EmptyMockComponent).apply(this, arguments));
    }

    return EmptyMockComponent;
})(React.Component);

var GoodMockComponent = (function (_React$Component2) {
    _inherits(GoodMockComponent, _React$Component2);

    function GoodMockComponent() {
        _classCallCheck(this, GoodMockComponent);

        return _possibleConstructorReturn(this, Object.getPrototypeOf(GoodMockComponent).apply(this, arguments));
    }

    return GoodMockComponent;
})(React.Component);

var BadMockComponent = (function (_React$Component3) {
    _inherits(BadMockComponent, _React$Component3);

    function BadMockComponent() {
        _classCallCheck(this, BadMockComponent);

        return _possibleConstructorReturn(this, Object.getPrototypeOf(BadMockComponent).apply(this, arguments));
    }

    return BadMockComponent;
})(React.Component);

lifeCycleMethods.forEach(function (methodName) {
    GoodMockComponent.prototype[methodName] = function (first, second) {
        return first + second;
    };

    BadMockComponent.prototype[methodName] = function () {
        throw new Error(methodName + ' Error');
    };
});

describe('The react-safe-component Test Suite', function () {
    lifeCycleMethods.forEach(function (methodName) {
        describe('When we get wrap the ' + methodName + '() method', function () {
            it('Should instantiate with the original arguments', function () {
                var Component = wrap(GoodMockComponent);
                var component = new Component();
                var value = component[methodName]('foo', 'bar');

                assert.equal(value, 'foobar');
            });

            it('Should not throw an error if there is one', function () {
                var Component = wrap(BadMockComponent);
                var component = new Component();

                assert.doesNotThrow(component[methodName], Error);
            });

            it('Should not stub the method if it does not exist', function () {
                var Component = wrap(EmptyMockComponent);
                var component = new Component();

                assert.ok(!component[methodName]);
            });

            if (methodName === 'shouldComponentUpdate') {
                it('Should return a boolean for componentShouldUpdate on error', function () {
                    var Component = wrap(BadMockComponent);
                    var component = new Component();

                    assert.equal(_typeof(component[methodName]()), 'boolean');
                });
            }

            if (methodName === 'render') {
                it('Should return a message wrapped in a react element if there is an error', function () {
                    var Component = wrap(BadMockComponent);
                    var renderer = ReactTestUtils.createRenderer();

                    renderer.render(React.createElement(Component));

                    var _renderer$getRenderOu = renderer.getRenderOutput();

                    var props = _renderer$getRenderOu.props;

                    assert.equal(props.className, 'react__safecomponent-error');
                    assert.equal(props.children, 'Oops... an error has occured');
                });

                it('Should return a custom error', function () {
                    BadMockComponent.prototype.renderSafeComponentError = function () {
                        return 'Custom Error!';
                    };

                    var Component = wrap(BadMockComponent);
                    var renderer = ReactTestUtils.createRenderer();

                    renderer.render(React.createElement(Component));

                    var _renderer$getRenderOu2 = renderer.getRenderOutput();

                    var props = _renderer$getRenderOu2.props;

                    assert.equal(props.className, 'react__safecomponent-error');
                    assert.equal(props.children, 'Custom Error!');
                });
            }
        });
    });
});