'use strict';

var React = require('react');
var lifeCycleMethods = require('./lifeCycleMethods');
var debug = require('debug')('react-safe-component');

var wrap = function wrap(Component) {

    if (!Component.prototype.renderSafeComponentError) {
        Component.prototype.renderSafeComponentError = function () {
            return 'Oops... an error has occured';
        };
    }

    var wrapMethod = function wrapMethod(methodName) {
        var originalMethod = Component.prototype[methodName];

        if (!originalMethod) {
            return;
        }

        Component.prototype[methodName] = function () {
            try {
                return originalMethod.apply(this, arguments);
            } catch (e) {
                debug(e);

                if (methodName === 'render') {
                    return React.createElement('div', {
                        className: 'react__safecomponent-error'
                    }, Component.prototype.renderSafeComponentError());
                }

                if (methodName === 'shouldComponentUpdate') {
                    return false;
                }
            }
        };
    };

    lifeCycleMethods.forEach(wrapMethod);

    return Component;
};

module.exports = wrap;